# this script uses 1 db key named "MIN_SHARE". This key contains the minimal shared size in bytes
# to add this key to the database, use "-dbadd MIN_SHARE str xxx" where xxx is the requirement.
# NOTE: the key type is string, not integer.
# the MIN_SHARE requirement is not used against hub OP or Master, only with standard user.

sub dchub_myinfo_handler
{
   local (*evt_array)=$_[0];
	print "myinfo handler\n";

   printf "event   = '".$evt_array{"event"}."'\n";
   printf "nickname= '".$evt_array{"nickname"}."'\n";
   printf "argc    = '".$evt_array{"argc"}."'\n";

	if($evt_array{"argc"}!=1)
	{
		dchub::disconnect($evt_array{"nickname"});
	}
	else
	{
		# $evt_array{"0"} has the following format:
		# "$MyINFO $ALL nickname userdescription$ $connexiontype%$email$sharesize$|"
		# % is a bitmask. bit0: always 1
		#                 bit1: 0=here,1=away
		#                 bit2: 0=normal, 1=client is a server (up for more than 2 hours, ...)
		#                 bit3: 0=normal, 1=client is fast (output greater than 100KB/s)
		# the format is quite painful to decode because, even if the space is the best separator,
		# it can appear in some fields, that's why the separator is $ or space 
		#my(@uinfo)=split(/\$/,$evt_array{"0"});
		#uinfo[0]=""
		#uinfo[1]="MyINFO "
		#uinfo[2]="ALL nickname userdescription"
		#uinfo[3]=" "
		#uinfo[4]="connexiontype%"
		#uinfo[5]="email"
		#uinfo[6]="sharedsize"
		#uinfo[7]="|"

		# a better solution is to use dchub::nickinfo which uses the $ as separator
		my(@uinfo)=split(/\$/,dchub::nickinfo($evt_array{"nickname"}));
		my $duration=dchub::nick_duration($evt_array{"nickname"});

		my $min_share=dchub::db_get("MIN_SHARE");
		if(defined($min_share))
		{
			if($uinfo[1]<$min_share)
			{
				my $str="You must share more than ";

				if($min_share<1024)
				{
					$str.="$min_share ";
				}
				elsif($min_share<(1024*1024))
				{
					$str.=($min_share/1024)." K";
				}
				elsif($min_share<(1024*1024*1024))
				{
					$str.=($min_share/(1024*1024))." M";
				}
				else
				{
					$str.=($min_share/(1024*1024*1024))." G";
				}
	
				$str.="bytes. Add some files and come back later.";

				if($uinfo[5]==0)
				{
					dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> $str|");
					dchub::disconnect($evt_array{"nickname"});
					#dchub::redirectto($evt_array{"nickname"},"192.168.1.1");
				}
				else
				{
					if($duration<4)
					{
						dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> $str ... hopefully, you are an op :) |");
					}
				}
			}
		}
		else
		{
			if($duration<4)
			{
				dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> This hub does not have minimum share requirement. Feel free to share what you want.|");
			}
		}
  				
		#dc++ version, number of hubs and slot's shared checker

		if($uinfo[3]=~/<\+\+\ V:.*,M:.*<\+\+\ V:.*,M:/) 
		{
		    dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> Kicking you for trying to fool the script, change your description. |");
		    dchub::disconnect($evt_array{"nickname"});
		}
		elsif($uinfo[3]=~/<\+\+\ V:.*,M:.,H:.*,S:.*>$/) 
		{
		    my $hub_max=dchub::db_get("HUB_MAX");         #you have to add HUB_MAX to the db and set it to the max number of hubs that a user can be at the same time
		    my $slot_min=dchub::db_get("SLOT_MIN");       #you have to add SLOT_MIN to the db and set to the min slots/hub that a user must have  
		    my $dc_ver=dchub::db_get("DC_V");		  #you have to add DC_V to the db and set it to the min dc++ version that a dc++ user must have   
		    @test=split(/:/, $uinfo[3]);
		    @HUBS=split(/,/, $test[3]);                   #$HUBS[0] is the number of hub's that the user is in                               
		    @SLOTS=split(/>/, $test[4]);                  #$SLOTS[0] is the number of slots that the user have shared
    		    @VER=split(/,/, $test[1]); 			  #$VER[0] is the version of dc++ that the user have
		    #the 2 lines below are for hub owners that dont want to allow user's with dc++, remove the # before the lines to auto kick dc++ users that connect    	
		    #dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> This hub dont allow use of DC++ change dc client. |"); 
		    #dchub::disconnect($evt_array{"nickname"});
		    #some info that i used to debug
		    #dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> you are runing DC++ version $VER[1] hub max is $hub_max hubs and you are in $HUBS[0] hubs, slot min is $slot_min slots/hub you have $SLOTS[0] slots |");

		    if(defined($slot_min) && defined($hub_max))
		    {
			$TOTAL=$slot_min*$HUBS[0];                #the number of slots_min*the number of hubs u are in = the number of slots you need to get in
	
			if($slot_min>$SLOTS[0])
			{
			    dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> You have to few slots shared(\ $SLOTS[0] )\ you must atleast share $slot_min slot's for every hub you are connected to. |");
			    dchub::disconnect($evt_array{"nickname"});	
		        }
			elsif($hub_max<$HUBS[0])
			{
			    dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> You are connected to to many hub's (\ $HUBS[0] )\ you may only be on max $hub_max hub's if you are going to get in here. |");
			    dchub::disconnect($evt_array{"nickname"});	
		        }
			elsif($TOTAL>$SLOTS[0])
			{
			
			    dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> You have to few slots for the number of hub's u are in u have to have atleast $slot_min st slots for every hub that u are connected to to get in here you have $SLOTS[0] slots you need $TOTAL. |");
			    dchub::disconnect($evt_array{"nickname"});	
			}
		    }

		    elsif(defined($hub_max))
		    {

			if($hub_max<$HUBS[0])
			{
			dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> You are connected to to many hub's (\ $HUBS[0] )\ you may only be on max $hub_max hub's if you are going to get in here. |");
			dchub::disconnect($evt_array{"nickname"});	
		        }
		    }

		    elsif(defined($slot_min))
		    {
			if($slot_min>$SLOTS[0])
			{
			    dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> You have to few slots shared(\ $SLOTS[0] )\ you must atleast share $slot_min slot's. |");
			    dchub::disconnect($evt_array{"nickname"});	
		        }
		    }
		    #my $dc_ver=dchub::db_get("DC_V");
		    if(defined($dc_ver))
		    {  			
			if($VER[0]<$dc_ver)
			{
			   dchub::send_to_named_user($evt_array{"nickname"},"<Hub-Security> You have to old version of dc++ upgrade to atleast $dc_ver. |");
			   dchub::disconnect($evt_array{"nickname"}); 
			}	    
		    }
		    
		}
	}
   print "#########################################################\n";
}
