/* DChub - a Direct Connect Hub for Linux
 * Copyright (C) 2001 Eric Prevoteau
 *
 * cnx_lst.h: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __CNX_LST_H__
#define __CNX_LST_H__

#include "hl_locks.h"
#include <netinet/in.h>

typedef struct s_GString_ref
{
  GString	*data;
  int		count;
}GString_ref;


GString_ref	*g_string_ref_new(const char *);
GString_ref	*conv_gstring_to_gstring_ref(GString *);
void		*g_string_ref_free(GString_ref *);



struct s_cnx_link;

typedef struct s_entry
{  
	int sock_fd;
	time_t start_time;			/* creation time of this thread */
	int handshake_done;			/* ==0, the thread is in handshake stage. ==1, handshake over, normal mode */
  int connected;				/* ==1 if the hub is connected else ==0 */

	/* DC users information */
	const char *user_cnx_type;
	GString *user_nick;
	unsigned long long shared_size;
	GString *user_mail;
	GString *user_description;
	unsigned int user_flag;
	int is_op;						/* is it a hub operator ? */
  /* privilege mask : 0=no
     OP_PRIV = operator
     MASTER_PRIV = master
     BOT_PRIV = bot
     HUB_PRIV = hub
     FUSION_PRIV = fusion */
	GString *client_version;

	GList *incoming_commands;	/* glist of g_string, each one containing a string to send */
										/* g_string stored here are ALWAYS ended by a '|' */
	GString *in_partial;			/* partially received string */
	GList *outgoing_commands;	/* glist of g_string, each one containing a string to send */
  struct in_addr ip;		/* user ip (added for in multi-server vertion)*/
  unsigned int server_id;		/* route ID for multi-server if =0 it's a local user*/
  char *created_lock;
  char *temp_nick;
  char *user_pass;
  int log_level;
  int tmp_op_type;
  time_t	last_hcall;
  time_t	last_msg;
  int	msg_count;
  int	sr_count;
  struct s_entry *next;
  struct s_cnx_link *next_on_chat;
  unsigned int in_data; /* 32bit may not be engnouth */
  unsigned int out_data;  /* 32bit may not be engnouth */
  unsigned int in_cnx_count; /* log the nuber of atemplt */
  unsigned int out_cnx_count; /* log the nuber of atemplt */
} CNX_ENTRY;

typedef struct s_cnx_hash_table
{
  CNX_ENTRY		**entry;
  CNX_ENTRY		*wait;
  int			len;
  int			locallen;
  int			wait_len;
}cnx_hash_table;

#define HASH_SIZE 523

/**********************************/
/* list of all running connexions */
/**********************************/
extern cnx_hash_table *cnx_lst;
extern HL_MUTEX cnx_struct_lock;

// extern PtrArray *cnx_lst;

/***************************************************/
/* create a copy of the given CNX_ENTRY and put it */
/* inside the cnx_lst.                             */
/******************************************************/
/* input: orig= entry to add the cnx_lst              */
/*        no_dup= 0, create a copy of orig and put the*/
/*                   copy in the cnx_lst.             */
/*              = 1, put orig itself in cnx_lst, don't*/
/*                   create copy.                     */
/* output: address of the newly created entry or NULL */
/******************************************************/
CNX_ENTRY *add_new_cnx_entry(CNX_ENTRY *orig, int no_dup);

/**************************************************************************************/
/* delete the given entry from the cnx_lst or the wait list and free allocated memory */
/**************************************************************************************/
/* input: address of the entry to remove */
/* output: 0=ok, 1=error                 */
/*****************************************/
int     remove_from_all_cnx_table(cnx_hash_table *table, CNX_ENTRY *ent);

/*********************************************************************/
/* delete the given entry from the cnx_lst and free allocated memory */
/*********************************************************************/
/* input: address of the entry to remove */
/* output: 0=ok, 1=error                 */
/*****************************************/
int delete_cnx_entry(CNX_ENTRY *ent);

/***************************************************************************/
/*   use delete_cnx_unlinked_entry when the cnx isen't linked from         */
/*   the hach table or que queue                                           */
/***************************************************************************/
/* input: address of the entry to remove */
/* output: 0=ok, 1=error                 */
/*****************************************/
int delete_cnx_unlinked_entry(CNX_ENTRY *ent);

/********************************************************/
/* append a copy of the given string to the given glist */
/********************************************************/
void append_new_cmd_to_glist_outgoing(CNX_ENTRY *ent,GList **pa, GString_ref *string);

/********************************************************/
/* append a copy of the given string to the given glist */
/********************************************************/
void append_new_cmd_to_glist_incoming(CNX_ENTRY *ent,GList **pa, char *string);

/*********************************************************/
/* prepend a copy of the given string to the given glist */
/*********************************************************/
void prepend_new_cmd_to_glist(CNX_ENTRY *ent,GList **pa, char *string);

/**************************************************************/
/* return the pointer of the first GString of the given GList */
/**************************************************************/
/* output: the pointer or NULL. the pointer and the GString */
/*         content should not be modified                   */
/************************************************************/
const GString_ref * const peek_first_string_of_glist_outgoing(CNX_ENTRY *ent, GList **pa);

const GString * const peek_first_string_of_glist_incoming(CNX_ENTRY *ent, GList **pa);

/************************************************************/
/* same as peek_first_string_of_glist except the GString is */
/* also removed from the GList. You must free it yourself   */
/* when you don't need it anymore.                          */
/************************************************************/
/* output: the pointer or NULL */
/*******************************/
GString_ref *take_first_string_of_glist_outgoing(CNX_ENTRY *ent, GList **pa);

GString *take_first_string_of_glist_incoming(CNX_ENTRY *ent, GList **pa);

/************************************/
/* send a string on the given entry */
/************************************/
#define send_str_to(ent,str)		append_new_cmd_to_glist_outgoing(ent,&(ent->outgoing_commands),str)

#define send_const_str_to(ent,str)		{\
					GString_ref *__local_str;\
					__local_str=g_string_ref_new(str); \
					append_new_cmd_to_glist_outgoing(ent,&(ent->outgoing_commands),__local_str);\
					g_string_ref_free(__local_str);}

#define send_srv_str_to(ent,str)		append_new_cmd_to_srv_glist(ent,&(ent->outgoing_commands),str)


#define queue_const_str_to_local_everyone(str)	{\
						GString_ref *__local_str;\
						__local_str=g_string_ref_new(str); \
						queue_to_local_everyone(__local_str);\
						g_string_ref_free(__local_str);}

#define queue_const_str_to_everyone(str)	{\
						GString_ref *__local_str;\
						__local_str=g_string_ref_new(str); \
						queue_to_everyone(__local_str);\
						g_string_ref_free(__local_str);}

#define queue_const_str_to_extern_everyone(str)	{\
						GString_ref *__local_str;\
						__local_str=g_string_ref_new(str); \
						queue_to_extern_everyone(__local_str);\
						g_string_ref_free(__local_str);}



/************************************************/
/* call send_str_to for all existing connexions */
/************************************************/
void queue_to_everyone(GString_ref *str);
void queue_to_everyone_exetp(GString_ref *str, const char *expt);;
void queue_to_extern_everyone(GString_ref *str, char* from);
void queue_to_local_everyone(GString_ref *str);
/****************************************************************/
/* call send_str_to for the connexion having the given nickname */
/****************************************************************/
CNX_ENTRY *send_str_to_named_user(const char *dest, char *string);

/***********************************************************************/
/* return the connetion having the given nickname or NULL if not exist */
/***********************************************************************/
/* WARNING: before using this function, you must lock cnx_lst */
/*          before and release the lock when no more useful.  */
/**************************************************************/
CNX_ENTRY *find_cnx_by_nickname(const char *nick);

/***************************/
/* get hub user statistics */
/***************************/
void get_hub_user_stat(unsigned long long *shared_size, unsigned int *nb_users);

/*************************************/
/* get the number of connected users */
/*************************************/
int get_nb_connected(void);

/********************************************/
/* get the number of local  connected users */
/********************************************/
int get_local_nb_connected(void);

/***************************/
/* kick the given nickname */
/**************************************************************/
/*  this function closes incoming side of the nickname socket */
/* and broadcast the Hub-Security message.                    */
/**************************************************************/
void kick_named_user(CNX_ENTRY *ptr,char *nickname);

/*********************************/
/* disconnect the given nickname */
/*********************************/
void do_disconnect(char *nickname);

/***************************************************/
/* get the user information for the given nickname */
/***************************************************/
/* output: NULL if user not found */
/**********************************/
GString *get_user_info(char *nick);

/******************************************************/
/* get the connection duration for the given nickname */
/**************************************************************/
/* output: 0= not found, else 1 and *duration is the duration */
/**************************************************************/
int get_user_connection_duration(char *nick, time_t *duration);

/********************************************************************/
/* retrieve the list of all connected users                         */
/* The list is returned as a string, each nickname separated by a $ */
/********************************************************************/
GString *get_userlist(void);

/********************************************************************/
/* retrieve the list of all connected ops                           */
/* The list is returned as a string, each nickname separated by a $ */
/********************************************************************/
GString *get_oplist(void);

/********************************************************************/
/* create a new hash table structure                                */
/********************************************************************/
cnx_hash_table	*new_cnx_hash_table(void);

/********************************************************************/
/* insert a cnx in the hash table                                   */
/********************************************************************/
/* WARNING: before using this function, you must lock cnx_lst */
/*          before and release the lock when no more useful.  */
/**************************************************************/
int     add_in_cnx_hash_table(cnx_hash_table *table, CNX_ENTRY *new);

/********************************************************************/
/* find a cnx by nick in the hash table                             */
/********************************************************************/
/* WARNING: before using this function, you must lock cnx_lst */
/*          before and release the lock when no more useful.  */
/**************************************************************/
CNX_ENTRY     *get_in_cnx_hash_table_by_nick(const cnx_hash_table *table, const char *str);

/********************************************************************/
/* remove a cnx by nick in the hash table                           */
/********************************************************************/
/* WARNING: before using this function, you must lock cnx_lst */
/*          before and release the lock when no more useful.  */
/**************************************************************/
int     del_in_cnx_hash_table_by_nick(cnx_hash_table *table, const char *str);

/********************************************************************/
/* add a cnx without nickname                                       */
/********************************************************************/
/* WARNING: before using this function, you must lock cnx_lst */
/*          before and release the lock when no more useful.  */
/**************************************************************/
int     add_in_cnx_wait_table(cnx_hash_table *table, CNX_ENTRY *new);

/********************************************************************/
/* insert a user in the cnx table                                   */
/********************************************************************/
/* WARNING: before using this function, you must lock cnx_lst */
/*          before and release the lock when no more useful.  */
/**************************************************************/
int     inject_wait_table(cnx_hash_table *table);

#endif
