/* DChub - a Direct Connect Hub for Linux
 * Copyright (C) 2001 Eric Prevoteau
 *
 * multi_private_chat.c: Copyright (C) Uriel Chemouni <uriel@adess.dyndns.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __MULTI_PUBLIC_CHAT_H__
#define __MULTI_PUBLIC_CHAT_H__

#include <string.h>
#include <glib.h>
#include "cnx_lst.h"

/******************************************/
/* check if the given name is a chat_name */
/******************************************/
/* output: 1=yes, 0=no */
/***********************/
int is_a_pubchat_name(const char *name);

int	cmd_list_multi_pubchat(const char *cmd, CNX_ENTRY *ptr,GString **dest_base, const GString *str, char *xtra_param, const GPtrArray *splitted_str);

/*******************************/
/* join a chan */
/* cmd == "+join "                                    */
/* ptr is the user making the command                 */
/* dest_base="$To: remotenick From: Hub-Security $"   */
/* str=the full incoming string as it as been entered */
/* xtra_param= NULL                                   */
/* splitted_str= str splitted along " "               */

int join_pubchat(const char *cmd, CNX_ENTRY *ptr,GString **dest_base, const GString *str, char *xtra_param, const GPtrArray *splitted_str);
int pubchat_user_quit(CNX_ENTRY *ptr);
int pubchat_user_leave(CNX_ENTRY *ptr, int chat_id);

/***********************************/
/* leave the given multi user chat */
/**************************************/
/* cmd syntax: "+leave_pubchat chatname */
/******************************************************/
/* cmd == "+leave_pubchat "                             */
/* ptr is the user making the command                 */
/* dest_base="$To: remotenick From: Hub-Security $"   */
/* str=the full incoming string as it as been entered */
/* xtra_param= NULL                                   */
/* splitted_str= str splitted along " "               */
/******************************************************************************/
/* the function must return >=0 if the connection must remain alive (no error */
/* (0) or recoverable error (>0)) and a value <0 if there is a fatal error)   */
/******************************************************************************/
int cmd_leave_multi_pubchat(const char *cmd, CNX_ENTRY *ptr,GString **dest_base, const GString *str, char *xtra_param, const GPtrArray *splitted_str);

void	append_public_chat_name(GString *ulst, CNX_ENTRY *ptr);
void	send_to_users_of_multi_pubchat_except2(int idx, char *base_msg, CNX_ENTRY *ptr);
int find_pubchat_idx_by_name(const char *chat_name);


#endif /* ! __MULTI_PUBLIC_CHAT_H__ */
