/* DChub - a Direct Connect Hub for Linux
 * Copyright (C) 2001 Eric Prevoteau
 *
 * passwd.c: Copyright (C) Eric Prevoteau <www@ac2i.tzo.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "gvar.h"
#include "passwd.h"
#include "toolkit.h"
#include <regex.h>

char	*fusion_pass = NULL;
char	*fusion_user = NULL;

/********************************************************/
/* verify if the username do not match the locked patern*/
/*********************************************************/
/* output: 1 if the name is lock else 0                  */
/*********************************************************/

int	is_lock_nick(const char *nick)
{
	if (gl_lock_patern == NULL)
		return (0);
	if (regexec(gl_lock_patern, nick, 0, NULL, 0) == 0) /* regexec return 0 if match */
		return (1);
	return (0);
}

/*************************************************/
/* get the password associated to the given nick */
/*************************************************************/
/* output: NULL if no password required (*op_type unchanged) */
/*         else, it is the password (must be free()d when no */
/*         more required. (*op_type) is set to the type of   */
/*         "account": | 0: normal user,                      */
/*                    | 1: operator,                         */
/*                    | 2: master of the hub                 */
/*                    | 4: Bot                               */
/*                    | 8: Fusion                           */
/*                    |16: hub not a user                    */
/*************************************************************/
char *get_user_password(char *nick, int *op_type)
{
	char *passwd=NULL;
	char buf[PASSWD_ENTRY];
	
	G_LOCK(user_file);
	rewind(user_file);
	while(fread(buf,1,PASSWD_ENTRY,user_file)==PASSWD_ENTRY)
	{
		if((buf[0]=='U')&&
			(!strcasecmp(nick,buf+NICK_PASS_POS)))
		{
			passwd=strdup(buf+PASS_PASS_POS);
			switch(((int)buf[TYPE_PASS_POS])&255)
			{
				case 'O':	*op_type=OP_PRIV;
								break;
				case 'M':	*op_type=OP_PRIV | MASTER_PRIV;
								break;
				case 'b':	*op_type=OP_PRIV | BOT_PRIV;
								break;
				case 'B':	*op_type=OP_PRIV | MASTER_PRIV | BOT_PRIV;
								break;
				case 'F':	*op_type=OP_PRIV | MASTER_PRIV | BOT_PRIV | FUSION_PRIV;
								break;
				default:
				case 'N':	*op_type=0;
								break;
			}
			goto done;
		}
	}

	done:
	G_UNLOCK(user_file);

	if (passwd == NULL)
	{
		if (is_lock_nick(nick))
			passwd = strdup("||||"); /* this password can not be use by the client then it's impossible to log in */
	}
	return passwd;
}

/**********************************************/
/* check if the given nickname has an account */
/**************************************************************/
/* output: 1=yes, 0=no. If op_type is not NULL and output==1, */
/* *op_type contains the type of "account"                    */
/*                   == 0: normal user,                       */
/*                    | 1: operator,                          */
/*                    | 2: master of the hub                  */
/*                    | 4: Bot                                */
/*                    | 8: Fusuion                            */
/*                    |16: hub not a user                     */
/**************************************************************/
int account_exist(char *nick, int *op_type)
{
	int ret=0;
	char buf[PASSWD_ENTRY];
	
	G_LOCK(user_file);
	rewind(user_file);
	while(fread(buf,1,PASSWD_ENTRY,user_file)==PASSWD_ENTRY)
	{
		if((buf[0]=='U')&&
			(!strcasecmp(nick,buf+NICK_PASS_POS)))
		{
			ret=1;
			if(op_type!=NULL)
			{
				switch(((int)buf[TYPE_PASS_POS])&255)
				{
				case 'O':	*op_type=OP_PRIV;
								break;
				case 'M':	*op_type=OP_PRIV | MASTER_PRIV;
								break;
				case 'b':	*op_type=OP_PRIV | BOT_PRIV;
									break;
				case 'B':	*op_type=OP_PRIV | MASTER_PRIV | BOT_PRIV;
								break;
				case 'F':	*op_type=OP_PRIV | MASTER_PRIV | BOT_PRIV | FUSION_PRIV;
									break;
					default:
					case 'N': *op_type=0;
						break;
				}
			}
			goto done;
		}
	}

	done:
	G_UNLOCK(user_file);

	return ret;
}

/******************************************************************/
/* create the list of all registered users with their information */
/******************************************************************/
/* the returned Gstring must be freed when no more useful */
/**********************************************************/
GString *full_user_list(void)
{
	GString *lst;
	char buf[PASSWD_ENTRY];
	int nb=0;
	int usr=0,op=0,master=0,bot=0,mbot=0;

	lst=g_string_new("T:Nickname:Password:\r\n");

	G_LOCK(user_file);
	rewind(user_file);
	while(fread(buf,1,PASSWD_ENTRY,user_file)==PASSWD_ENTRY)
	{
		if(buf[0]=='U')
		{
			g_string_sprintfa(lst,"%c:%.32s:%.16s:\r\n",buf[TYPE_PASS_POS],buf+NICK_PASS_POS,buf+PASS_PASS_POS);
			nb++;
			switch(((int)buf[TYPE_PASS_POS])&255)
			{
				case 'O':	op++;
								break;
				case 'M':	master++;
								break;
				case 'b':	bot++;
								break;
				case 'B':	mbot++;
								break;
				default:
				case 'N':	usr++;
								break;
			}
		}
	}

	G_UNLOCK(user_file);

	g_string_sprintfa(lst,"%d accounts exists (users: %d, operator: %d, bot: %d, master-bot: %d, master: %d)\r\n",
								nb,usr,op,bot,mbot,master);
	return lst;
}

/***************************************/
/* add a new password to the user list */
/***************************************/
/* output: 0=ok, !=0=error */
/********************************************************************/
/* note: this function doesn't perform any check of the given value */
/********************************************************************/
int add_user_to_ulist(char *nick, char *password, char account_type)
{
	char buf[PASSWD_ENTRY];
	long cur_pos;
	int ret=0;
	
	char nw_buf[PASSWD_ENTRY];

	/* create the new entry */
	nw_buf[0]='U';		/* entry is busy */
	strncpy_max(nw_buf+NICK_PASS_POS,nick,NICK_PASS_LEN);
	strncpy_max(nw_buf+PASS_PASS_POS,password,PASS_PASS_LEN);
	nw_buf[TYPE_PASS_POS]=account_type;

	/* and write it into the file */
	G_LOCK(user_file);
	rewind(user_file);
	cur_pos=ftell(user_file);
	while(fread(buf,1,PASSWD_ENTRY,user_file)==PASSWD_ENTRY)
	{
		if(buf[0]=='F')
		{
			/* empty entry found, replace it */
			fseek(user_file,cur_pos,SEEK_SET);
			fwrite(nw_buf,1,PASSWD_ENTRY,user_file);
			goto done;
		}
		cur_pos=ftell(user_file);
	}

	/* not empty entry found, seek to the latest valid position */
	fseek(user_file,cur_pos,SEEK_SET);
	if(fwrite(nw_buf,1,PASSWD_ENTRY,user_file)!=PASSWD_ENTRY)
		ret=1;

	done:
	G_UNLOCK(user_file);

	return ret;
}

/************************************/
/* delete a user from the user list */
/************************************/
/* output: 0=ok, !=0=error */
/********************************************************************/
/* note: this function doesn't perform any check of the given value */
/********************************************************************/
int del_user_to_ulist(char *nick)
{
	char buf[PASSWD_ENTRY];
	long cur_pos;
	int ret=0;
	
	/* delete the entry */
	G_LOCK(user_file);
	rewind(user_file);
	cur_pos=ftell(user_file);
	while(fread(buf,1,PASSWD_ENTRY,user_file)==PASSWD_ENTRY)
	{
		if((buf[0]=='U')&&
			(!strcasecmp(nick,buf+NICK_PASS_POS)))
		{
			/* entry found, clear it */
			buf[0]='F';
			fseek(user_file,cur_pos,SEEK_SET);
			fwrite(buf,1,PASSWD_ENTRY,user_file);
			goto done;
		}
		cur_pos=ftell(user_file);
	}

	/* entry not found */
	ret=1;

	done:
	G_UNLOCK(user_file);

	return ret;
}

/*****************************************/
/* rename the given oldnick into newnick */
/*****************************************/
/* output: 0=ok, !=0=error */
/***************************************************************/
/* note: this function doesn't check if newnick already exists */
/***************************************************************/
int rename_user_to_ulist(char *oldnick,char *newnick)
{
	char buf[PASSWD_ENTRY];
	long cur_pos;
	int ret=0;
	
	/* update the existing entry */
	G_LOCK(user_file);
	rewind(user_file);
	cur_pos=ftell(user_file);
	while(fread(buf,1,PASSWD_ENTRY,user_file)==PASSWD_ENTRY)
	{
		if((buf[0]=='U')&&
			(!strcasecmp(oldnick,buf+NICK_PASS_POS)))
		{
			/* entry found, clear it */
			strncpy_max(buf+NICK_PASS_POS,newnick,NICK_PASS_LEN);
			fseek(user_file,cur_pos,SEEK_SET);
			fwrite(buf,1,PASSWD_ENTRY,user_file);
			goto done;
		}
		cur_pos=ftell(user_file);
	}

	/* entry not found */
	ret=1;

	done:
	G_UNLOCK(user_file);

	return ret;
}

/*********************************************/
/* change the password of the given nickname */
/*********************************************/
/* output: 0=ok, !=0=error */
/***************************/
int chg_user_passwd_to_ulist(char *nick,char *newpasswd)
{
	char buf[PASSWD_ENTRY];
	long cur_pos;
	int ret=0;
	
	/* update the existing entry */
	G_LOCK(user_file);
	rewind(user_file);
	cur_pos=ftell(user_file);
	while(fread(buf,1,PASSWD_ENTRY,user_file)==PASSWD_ENTRY)
	{
		if((buf[0]=='U')&&
			(!strcasecmp(nick,buf+NICK_PASS_POS)))
		{
			/* entry found, clear it */
			strncpy_max(buf+PASS_PASS_POS,newpasswd,PASS_PASS_LEN);
			fseek(user_file,cur_pos,SEEK_SET);
			fwrite(buf,1,PASSWD_ENTRY,user_file);
			goto done;
		}
		cur_pos=ftell(user_file);
	}

	/* entry not found */
	ret=1;

	done:
	G_UNLOCK(user_file);

	return ret;
}

/*****************************************/
/* change the type of the given nickname */
/*****************************************/
/* output: 0=ok, !=0=error */
/***************************/
int chg_user_type_to_ulist(char *nick,char newtype)
{
	char buf[PASSWD_ENTRY];
	long cur_pos;
	int ret=0;
	
	/* update the existing entry */
	G_LOCK(user_file);
	rewind(user_file);
	cur_pos=ftell(user_file);
	while(fread(buf,1,PASSWD_ENTRY,user_file)==PASSWD_ENTRY)
	{
		if((buf[0]=='U')&&
			(!strcasecmp(nick,buf+NICK_PASS_POS)))
		{
			/* entry found, clear it */
			buf[TYPE_PASS_POS]=newtype;
			fseek(user_file,cur_pos,SEEK_SET);
			fwrite(buf,1,PASSWD_ENTRY,user_file);
			goto done;
		}
		cur_pos=ftell(user_file);
	}

	/* entry not found */
	ret=1;

	done:
	G_UNLOCK(user_file);

	return ret;
}
